using Microsoft.Win32;
using System;
using System.Windows.Forms;

namespace EventNoficationPlugin
{
    [System.Runtime.InteropServices.ProgId("EventNoficationPlugin")]
    public partial class MainInterface : UserControl
    {
        private readonly System.Timers.Timer mTimer1s = new System.Timers.Timer();

        private StateNotifier mStateNotifier;
        private MediaCenter.MCAutomation mMediaCenterAutomation = null;
        private int mCurrentKey = -1;

        /// <summary>
        /// This is the main constructor for the plugin
        /// </summary>
        public MainInterface()
        {
            InitializeComponent();
        }

        /// <summary>
        /// After the plugin has been created Media Center will call the following method, 
        /// giving us a reference to the Media Center interface.
        /// </summary>
        /// <param name="mediaCenterReference">
        /// Media Center Reference
        /// </param>        
        public void Init(MediaCenter.MCAutomation mediaCenterReference)
        {
            try
            {
                string notifyHost;
                int notifyPort;
                LoadConfigOptions(out notifyHost, out notifyPort);
                mStateNotifier = new StateNotifier(notifyHost, notifyPort);

                mMediaCenterAutomation = mediaCenterReference;
                mStateNotifier.SetPlaybackMode(mMediaCenterAutomation.GetCurPlaylist().Continuous,
                    mMediaCenterAutomation.GetCurPlaylist().GetCanPlayNext());
                mMediaCenterAutomation.FireMJEvent += MediaCenterAutomation_FireMJEventHandler;

                mTimer1s.Elapsed += (sender, e) => OnTimedEvent1s();
                mTimer1s.Interval = 1000;
            }
            catch (Exception e)
            {
                MessageBox.Show("A Fatal error has occured while creating plugin:-" + e.Message +
                        "\n The Failure Occured" +
                        "\n In Class Object " + e.Source +
                        "\n when calling Method " + e.TargetSite +
                        "\n \n The following Inner Exception was caused" + e.InnerException +
                        "\n \n The Stack Trace Follows: \n\n" + e.StackTrace);
                Enabled = false;
            }
        }

        private void LoadConfigOptions(out string notifyHost, out int notifyPort)
        {
            notifyHost = "127.0.0.1";
            notifyPort = 24728;

            using (RegistryKey Key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\J. River\Media Jukebox\Plugins\Interface\EventNoficationPlugin"))
            {
                if (Key != null)
                {
                    int newPort = (int)Key.GetValue("NotifyPort");
                    if (newPort > 1024 && newPort < 65535)
                        notifyPort = newPort;
                    string newHost = (string)Key.GetValue("NotifyHost");
                    if (!string.IsNullOrEmpty(newHost))
                        notifyHost = newHost;
                }
            }
        }

        private void OnTimedEvent1s()
        {
            mStateNotifier.SetPosition(mMediaCenterAutomation.GetPlayback().Position);

            mStateNotifier.SetPlaybackMode(mMediaCenterAutomation.GetCurPlaylist().Continuous,
                    mMediaCenterAutomation.GetCurPlaylist().GetCanPlayNext());
        }

        private void MediaCenterAutomation_FireMJEventHandler(string bstrType, string bstrParam1, string bstrParam2)
        {
            if (!Enabled || bstrType != "MJEvent type: MCCommand")
                return;

            switch (bstrParam1)
            {
                case "MCC: NOTIFY_TRACK_CHANGE": // Fired when a new track starts to play
                    HandleTrackChange();
                    break;
                case "MCC: NOTIFY_PLAYERSTATE_CHANGE": // Fired when a new track starts to play or playback is stopped/paused/started
                    HandleStateChange();
                    break;
                case "MCC: NOTIFY_PLAYLIST_ADDED": // Fired when a playlist is added by the user
                case "MCC: NOTIFY_PLAYLIST_INFO_CHANGED": // Fired when a playlist is renamed, smartlist rule is changed, or station is added to My Stations
                case "MCC: NOTIFY_PLAYLIST_REMOVED": // Fired when a playlist is removed
                case "MCC: NOTIFY_PLAYLIST_COLLECTION_CHANGED": // Fired when a playlist is moved or copied
                case "MCC: NOTIFY_PLAYLIST_FILES_CHANGED": // Fired when a playlist's files change or a service notifies Media Center that  playlists need to be refreshed
                case "MCC: NOTIFY_PLAYLIST_PROPERTIES_CHANGED": // Fired when a playlist property is changed
                case "MCC: NOTIFY_SKIN_CHANGED": // Fired when the skin is changed
                case "MCC: NOTIFY_VOLUME_CHANGED": // Fired when the volume is changed
                case "MCC: NOTIFY_EQ_CHANGED": // Fired when the Equalizer settings are changed, or the Equalizer is turned on or off.
                default:
                    // ignore
                    break;
            }
        }

        private void HandleTrackChange()
        {
            MediaCenter.IMJFileAutomation currentTrack = mMediaCenterAutomation.GetZones().GetZone(0).GetPlayingFile();
            if (currentTrack == null || mCurrentKey == currentTrack.GetKey())
                return;

            mCurrentKey = currentTrack.GetKey();

            if (currentTrack.Name == "Ipc")
            {
                mStateNotifier.SetTitle("External");
                mStateNotifier.SetDuration(0);
            }
            else
            {
                mStateNotifier.SetDuration(currentTrack.Duration);
                string mediaType = currentTrack.Get("Media Type", false);
                mStateNotifier.SetMediaType(mediaType);
                switch (mediaType)
                {
                    case "Video":
                    case "Image":
                    case "TV":
                        mStateNotifier.SetTitle(currentTrack.Name);
                        break;
                    case "Audio":
                        mStateNotifier.SetTitle(currentTrack.Artist + " - " + currentTrack.Album + " - " + currentTrack.Name);
                        break;
                    case "Data":
                    default:
                        mStateNotifier.SetTitle("None");
                        break;
                }

            }
        }

        private void HandleStateChange()
        {
            switch (mMediaCenterAutomation.GetPlayback().State)
            {
                case MediaCenter.MJPlaybackStates.PLAYSTATE_STOPPED:
                    mTimer1s.Enabled = false;
                    mStateNotifier.SetStopped();
                    break;
                case MediaCenter.MJPlaybackStates.PLAYSTATE_PLAYING:
                    mTimer1s.Enabled = true;
                    mStateNotifier.SetPlaying();
                    break;
                case MediaCenter.MJPlaybackStates.PLAYSTATE_PAUSED:
                    mTimer1s.Enabled = false;
                    mStateNotifier.SetPaused();
                    break;
                case MediaCenter.MJPlaybackStates.PLAYSTATE_WAITING:
                default:
                    // ignore
                    break;
            }
        }

    }
}