﻿using System;
using System.Net.Sockets;
using System.Text;

namespace EventNoficationPlugin
{
    internal sealed class StateNotifier
    {
        private readonly string mHost;
        private readonly int mPort;
        private readonly System.Timers.Timer mSendNotificationTimer = new System.Timers.Timer();

        // Information that we send
        private int mDuration = 0;
        private string mMediaType = string.Empty;
        private string mRepeatState = "off";
        private double mStartTime = 0;
        private int mStartPosition = 0;
        private string mState = "stop";
        private string mTitle = "None";

        public StateNotifier(string host, int port)
        {
            mHost = host;
            mPort = port;

            mSendNotificationTimer.Elapsed += (sender, e) => SendNotification();
            mSendNotificationTimer.Interval = 100;
            mSendNotificationTimer.AutoReset = false;
        }

        public void SetDuration(int duration)
        {
            if (mDuration == duration)
                return;

            mDuration = duration;
            mSendNotificationTimer.Enabled = true;
        }

        public void SetMediaType(string mediaType)
        {
            if (mMediaType == mediaType)
                return;

            mMediaType = mediaType;
            mSendNotificationTimer.Enabled = true;
        }

        public void SetPaused()
        {
            if (mState == "pause")
                return;

            mState = "pause";
            mSendNotificationTimer.Enabled = true;
        }

        public void SetPlaying()
        {
            if (mState == "play")
                return;

            mState = "play";
            mStartTime = DateTime.UtcNow.Subtract(new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc)).TotalMilliseconds;
            mSendNotificationTimer.Enabled = true;
        }

        public void SetPlaybackMode(bool isContinuous, bool canPlayNext)
        {
            string oldRepeatState = mRepeatState;

            if (isContinuous)
            {
                if (canPlayNext)
                    mRepeatState = "all";
                else
                    mRepeatState = "stop";
            }
            else
            {
                mRepeatState = "off";
            }

            if (oldRepeatState != mRepeatState)
                mSendNotificationTimer.Enabled = true;
        }

        private int mPosition = 0;
        public void SetPosition(int newPos)
        {
            if (newPos > 0)
            {
                if (!AreClose(mPosition, newPos, 5))
                {
                    mStartTime = DateTime.UtcNow.Subtract(new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc)).TotalMilliseconds;
                    mStartPosition = newPos;
                    mSendNotificationTimer.Enabled = true;
                }

                if (mPosition != newPos)
                    mPosition = newPos;
            }
        }

        private bool AreClose(int value1, int value2, int maxDifference)
        {
            return (value1 <= value2 + maxDifference) && (value1 >= value2 - maxDifference);
        }

        public void SetStopped()
        {
            if (mState == "stop")
                return;

            mState = "stop";
            mSendNotificationTimer.Enabled = true;
        }

        public void SetTitle(string title)
        {
            if (mTitle == title)
                return;

            mTitle = title;
            mSendNotificationTimer.Enabled = true;
        }

        private string mLastStateSent = string.Empty;
        private void SendNotification()
        {
            try
            {
                string state = ToJson();
                if (state != mLastStateSent)
                {
                    using (TcpClient tcpClient = new TcpClient(mHost, mPort))
                    using (NetworkStream clientStream = tcpClient.GetStream())
                    {
                        byte[] statusMessage = Encoding.ASCII.GetBytes(state);
                        clientStream.Write(statusMessage, 0, statusMessage.Length);
                    }
                }
            }
            catch { }
        }

        private string ToJson()
        {
            StringBuilder builder = new StringBuilder();

            builder.Append("{\"track_title\": \"");
            builder.Append(mTitle);
            builder.Append("\", \"start_time\": \"");
            builder.Append(mStartTime);
            builder.Append("\", \"start_position\": \"");
            builder.Append(mStartPosition);
            builder.Append("\", \"media_type\": \"");
            builder.Append(mMediaType);
            builder.Append("\", \"duration\": \"");
            builder.Append(mDuration);
            builder.Append("\", \"play_state\": \"");
            builder.Append(mState);
            builder.Append("\", \"repeat_state\": \"");
            builder.Append(mRepeatState);
            builder.Append("\"}");

            return builder.ToString();
        }

    }
}
